%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%
% Developed by:			Torbjrn Wigren
%                       Division of Systems and Control
%                       Department of Information Technology
%                       Uppsala University
%                       P O Box 337
%                       SE-751 05 Uppsala
%                       SWEDEN
%
% Version:					1.0, Quantizer
%
% Date:		        		rev. 1.0, April 10, 2007
%
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%
%                       DESCRIPTION
%
% This M-file creates a data vector consisting of input and nonlinear noisy
% outputs.
%
% First a discrete time time transfer function model (system description) 
% is simulated using a user chosen input. Then a nonlinear transformation 
% is applied and noise is added.
%
% The model used is a finite impulse response system. The static nonlinearity
% is an arbitrary quantizer. 
%
% Variables:        ts = sampling time
%                   b0dis = discrete time system numerator
%                   a0dis = discrete time system denominator
%                   tu = basic clock period of input (times sampling time)
%                   amp = amplitude of input signal / uniform distribution
%                   bias = bias in input signal
%                   n = length of data vector to be created
%                   u = input signal vector
%                   y = linear output vector
%                   fy = nonlinearly transformed linear output
%                   yn = nonlinear measurement signal with additive noise
%                   udummy = dummy variable
%                   lastrand = dummy variable
%                   i = loopcounter
%                   bw = bandwith of first order continuous time noise model
%                   r1 = variance of continuous noise model
%                   d0dis = numerator of sampled noise model
%                   c0dis = denominator of sampled noise model
%                   e = white input noise to drive the noise filter
%                   w = noise to be added to nonlinear output
%                   systtype = 0 if Wiener, 1 if static
%                   input_type = type of input signal
%
% The following parameters must always be present in the working area:
%                    
%                   input_type = type of input signal
%                   u = vector of input samples if input_type = 4
%
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%

%
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%  Setting system parameters.
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%
b0dis=[0 0.4 1.0];
a0dis=[1 0 0];
quantGrid=[0.5];
quantLevel=[-1.0 1.0];
ts=0.075;
bias=0.0;
amp=1.0;
tu=1;
if input_type < 4
   tu=6;
   n=2500;
end
systtype=0;
bw=10;
r1=0.05;
%
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
% Next an input is created. Several alternatives are available. Choose one
% by specifying the variable input_type. 
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%
if input_type==1
%
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
% Approximated PRBS.
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%
  u=amp*sign(randn(1,ceil(n/tu)));
  for i=1:tu-1
    u=[u;u(1:1, :)];
  end
  u=u(:);
end;
if input_type==2
%
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
% Uniformly distributed white noise input.
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%
  u=2*amp*rand(1,ceil(n/tu));
  for i=1:tu-1
    u=[u;u(1:1, :)];
  end;
  u=u(:)-amp;
end;
if input_type==3
%
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
% Approximated PRBS that is then given uniformly distributed amplitudes
% in each time interval of constant signal level.
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%
  udummy=sign(randn(1,ceil(n/tu)));
  for i=1:tu-1
    udummy=[udummy;udummy(1:1, :)];
  end
  lastrand=rand(1);
  u=amp*lastrand*udummy(:,1);
  for i=2:ceil(n/tu)
    if udummy(1,i)-udummy(1, (i-1)) == 0
      u=[u;lastrand*amp*udummy(:,i)];
    else
      lastrand=rand(1);
      u=[u;lastrand*amp*udummy(:,i)];
    end;
  end;
end;
if input_type==4
%
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
% In this case the input signal is given
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%
  n=length(u);
  tu=1;
end;
%
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
% After generating the input, the output of the linear part of the system
% is generated using digital filtering. First the operating point is
% however determined by adding the bias of the input signal, unless 
% the input_type = 4
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%
if input_type < 4
  u=u+bias;
end;
y=filter(b0dis,a0dis,u);
%
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
% Next a nonlinear transformation is applied to the output y.
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%
fy=zeros(n,1);
m=size(quantGrid,2);
for i=1:n
   fy(i)=quantLevel(1,1);
   for j=1:m
      if y(i)>quantGrid(1,j)
         fy(i)=quantLevel(1,j+1);
      end
   end
end
%
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
% The last thing to do is to add white measurement noise to the nonlinear output.
% To do this we use a first order continuous time (sampled) noise model where
% we can vary the variance and bandwith. The total energy in the noise is left
% constant when the bandwith is varied.
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%
d0dis=[1];
c0dis=[1;-exp(-bw*ts)];
e=r1*sqrt((1-exp(-2*bw*ts)))*randn(size(fy,1),1);
w=filter(d0dis,c0dis,e);
yn=fy+w;
%
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
% Clear variables that are not necessary for other programs.
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%
if input_type==3
  clear udummy;
  clear lastrand;
end;
clear d0dis;
clear c0dis;
clear y;
clear fy;
clear e;
clear w;
clear i;

