%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%
% Developed by:			Torbjrn Wigren
%                       Division of Systems and Control
%                       Department of Information Technology
%                       Uppsala University
%                       P O Box 337
%                       SE-751 05 Uppsala
%                       SWEDEN
%
% Version:					1.0, quantization
%
% Date:		        		rev. 1.0, April 10, 2007
%
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%
%                       DESCRIPTION
%
% This M-file implements the algorithm that identifies the linear part
% of a nonlinear Wiener type model when the output nonlinearity is an
% arbitrary quantizer, known priori. The gradient is approximated with 
% a filtered regression vector containing delayed inputs and model outputs, 
% as well as with a constant k_0 for the approximation of the gradient of 
% the known quantizer. This program is based on algorithm 1 of Wigren 1990.
%
% The quantizer shall be implemented in the function eval_fn.
%
% Functions called:  eval_fn
%
% Variables:         n = number of samples 
%                    u = input signal 
%                    yn = output signal 
%                    algnr = number of algorithm (=1)
%                    ymod = output from the linear part of the model	
%                    fymod = output from the nonlinear part of the model
%                    ynmod = output from the total nonlinear model (=fymod)
%                    a = identified A-polynomial
%                    b = identified B-polynomial
%		               d = prefilter for regressor filtering
%		               na = order of A-polynomial
%		               nb = order of B-polynomial
%		               nd = order of D-polynomial (normally = na)
%                    k0 = Scalefactor related to slope of nonlinearity
%		               phi = regression vector
%                    deltaphi = vector used in updating of phi
%		               phifilt = filtered regression vector
%		               phifilty = filtered regression vector (y-part)
%                    phifiltu = filtered regression vector (u-part)
%		               thetaoe = parameter vector containing a and b
%		               theta0 = initial value of parameter vector
%		               thetapro = parameter vector to be projected into Ds
%		               thetad = parameters of prefilter
%		               lambda = gain sequence
%		               perr = nonlinear prediction error
%		               S = scalar in Riccattieequation
%		               L = gain vector
%		               P = P-matrix in Riccattieequation
%                    a0 = initial value of a-polynomial
%		               b0 = initial value of b-polynomial
%		               lambda0 = rate of gain sequence (0.99 in L&S)
%		               zerovec = vector used to update phi
%		               delayy = matrix used to update phi
%		               delayu = matrix used to update phi
%		               delay = matrix composed of delayy and delayu
%                    theta = identified parameter vector
%                    nf = order of estimated denominator polynomial (=nb)
%                    F_0 = initial value of denominator polynomial
%                    B_0 = initial value of numerator polynomial
%                    lambda_0 = initial value of gain sequence ( 0.95 in L&S )
%                    P_0 = initial value of P-matrix
%
% The following variables must be present in the working area before 
% the program is run (i.e user initializations or data):
%
%                    n = number of data samples
%                    u = vector containing input signal samples
%                    yn = vector containing output signal samples
%                    F_0 = initial value of denominator polynomial (discrete)
%                    B_0 = initial value of numerator polynomial (discrete)
%                    lambda0 = rate of gain sequence
%                    lambda_0 = initial value of gain sequence
%                    P_0 = initial value of P-matrix
%                    k0 = scale factor related to slope of nonlinearity
% 
% Note that the d-polynomial is automatically choosen equal to F_0. Also
% note that numerator and denominator implicitely have the same order.
%
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%

%
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
% User initializations.
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%
nf=length(F_0)-1;
na=nf;
nb=length(B_0)-1;% Added handling of FIR case
if na>0 % Added handling of FIR case
   a0=F_0;
end
b0=B_0;
lambda=lambda_0;
P=P_0;
%
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
% Further standard initializations.
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%
algnr=1;
nd=na;
if na>0 % Added handling of FIR case
   a=a0;
end
b=b0;
if na>0  % Added handling of FIR case
   theta0=[a0(2:na+1)';b0(2:nb+1)'];
else
   theta0=[b0(2:nb+1)'];
end
%
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
% The d-polynomial is to be an a-priori guess of a0dis so a rather obvious
% choice is
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%
if na>0 % Added handling of FIR case
	d=a0;
   thetad=d(2:nd+1)';
end
%
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
% It remains to give initial values to phi. Since phi consists of inputs and
% delayed linear model outputs the result is
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%
phi=zeros(na+nb,1);
phifilt=phi;
%
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
% Also the time variables needs initial values.
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%
perr=zeros(n,1);
thetaoe=zeros(n,na+nb);
ufilt=zeros(n,1);
yfilt=zeros(n,1);
ymod=zeros(n,1);
ynmod=zeros(n,1);
%
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
% This ends the initialization.
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
% To efficiently update phi a zerovector and a delay matrix has to be created.
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%
zerovec=zeros(na+nb,1);
if na>0 % Added handling of FIR case
	if na>1
  		delayy=[[zeros(1,na-1);eye(na-1)]';zeros(1,na)]';
	else
  		delayy=0;
	end
	if nb>1
  		delayu=[[zeros(1,nb-1);eye(nb-1)]';zeros(1,na)]';
	else
  		delayu=0;
	end
   delay=[[delayy';zeros(nb,na)]';[zeros(na,nb);delayu']'];
else % Added handling of FIR case
	if nb>1
  		delayu=[[zeros(1,nb-1);eye(nb-1)]';zeros(1,nb)]';
	else
  		delayu=0;
	end
   delay=[[delayu']'];
end
%
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
% Now the recursive algorithm starts. t is the time.
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
t=1;
while t < n
%
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
% Compute prediction errors using prediction from previous iteration and
% save the result for later use in e.g. plotting routines.
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%
  perr(t)=yn(t)-ynmod(t);
%
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
% Then a new lambda-value is computed.
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%
  lambda=lambda0*lambda+1-lambda0;
%
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
% The next step is to iterate the Riccattieequation.
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%
  S=phifilt'*P*phifilt+lambda;
  P=(P-P*phifilt*phifilt'*P/S)/lambda;
  L=P*phifilt;
%
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
% Next the thetaoe-vector is updated and saved for later use. To get the same
% time indices for u, yn, ynmod, perr, d and thetaoe we distinguish between the
% cases t=1 and t>1. Projection into the stability set is used.
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%
  if t==1
    thetapro=theta0+L*perr(t);
  end;
  if t>=2
    thetapro=thetaoe(t-1,:)'+L*perr(t);
  end;
%
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
% Stability check.
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%
  if na>0 % Added handling of FIR case
	  	a=[1;thetapro(1:na)]';
     	if all(abs(roots(a))<1)
    		 thetaoe(t,:)=thetapro';
     	end;
     	if any(abs(roots(a))>=1)
       	 thetaoe(t,:)=thetaoe(t-1,:);
      end;
   else % Added handling of FIR case 
      thetaoe(t,:)=thetapro';
   end;   
%
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
% The model output at time t+1 is computed using the parametervector at
% time t. This means that phi has to be updated. This is done first.
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%
  deltaphi=zerovec;
  deltaphi(1)=-ymod(t);
  deltaphi(na+1)=u(t);
  phi=delay*phi+deltaphi;
  ymod(t+1)=thetaoe(t,:)*phi;
%
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
% The static nonlinearity is evaluated by a calling eval_fn.
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%
  [fymod,not_used]=eval_fn(ymod(t+1));
%
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
% Model output.
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
% 
  ynmod(t+1)=fymod;
%
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
% The last thing to do in the loop is to compute the gradient in this case
% phifilt. This is obtained by introducing the two auxiliary varibles yfilt
% and ufilt.
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%
	if na>0 % Added handling of FIR case
 		 if t<nd
   	    yfilt(t+1)=-thetad'*[yfilt(t:-1:1);zeros(nd-t,1)]+ymod(t+1);
    		 ufilt(t+1)=-thetad'*[ufilt(t:-1:1);zeros(nd-t,1)]+u(t+1);
  	    else
    		 yfilt(t+1)=-thetad'*yfilt(t:-1:t-nd+1)+ymod(t+1);
    		 ufilt(t+1)=-thetad'*ufilt(t:-1:t-nd+1)+u(t+1);
       end;
    else % Added handling of FIR case
       yfilt(t+1)=ymod(t+1);
       ufilt(t+1)=u(t+1);
    end
%
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
% After creating yfilt and ufilt it is a simple matter to create the filtered
% regression vector at time t+1.
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%
  	if na>0 % Added handling of FIR case 		  
  	  	if t<na
     		phifilty=[yfilt(t:-1:1);zeros(na-t,1)];
  		else
   		phifilty=yfilt(t:-1:t-na+1);
  		end;
	   if t<nb
    		phifiltu=[ufilt(t:-1:1);zeros(nb-t,1)];
  		else
    		phifiltu=ufilt(t:-1:t-nb+1);
  		end;
        phifilt=k0*[-1*phifilty;phifiltu];
   else % Added handling of FIR case
	   if t<nb
    		phifiltu=[ufilt(t:-1:1);zeros(nb-t,1)];
  		else
    		phifiltu=ufilt(t:-1:t-nb+1);
  		end;
        phifilt=k0*[phifiltu];
   end        
%
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
% That completes the loop.
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%
  t=t+1;
end
%
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
% Clear variables not used by subsequent programs.
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%
clear fymod;
clear a;
clear b;
clear d;
clear nd;
clear phi;
clear deltaphi;
clear phifilt;
clear phifilty;
clear phifiltu;
clear thetapro;
clear thetad;
clear S;
clear L;
clear zerovec;
clear delayu;
clear delayy;
clear delay;
clear not_used;
clear ymod;
clear theta0;
clear yfilt;
clear ufilt;
clear t;
clear lambda;
%
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
% Write parameter vector.
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%
theta=thetaoe(n-1,:);
theta
