%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%
% Developed by:			Torbjrn Wigren
%                       Division of Systems and Control
%                       Department of Information Technology
%                       Uppsala University
%                       P O Box 337
%                       SE-751 05 Uppsala
%                       SWEDEN
%
% Version:					1.0, PC MATLAB
%                       2.0, MATLAB 5
%
% Date:		        		rev. 1.0, Nov 10 1991
%                       rev. 2.0, March 28 2007
%
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%
%                       DESCRIPTION
%
% This M-file plots pole-zero diagrams of the estimated linear part
% of the Wiener model to check for a possible overparametrization.
%
% Variables:		      n = number of data samples (must exist)
%                       thetaoe = parameter vector
%                       poles = vector containing poles
%                       po_rad = radii of poles
%                       po_ang = angles of poles
%                       zeros_ = vector containing zeros
%                       ze_rad = radii of zeros
%                       ze_ang = angles of zeros
%                       max_radius = maximum modulus of any pole and zero
%                       ang = variable to draw axis 
%                       rad = variable to draw axis
%                       rad_max = variable to draw axis
%                       algnr = number of algorithm
%                       unit_circle = vector containing unit circle plot
%                       w = frequencies around the unit circle.
%                       na = order of denominator
%                       nb = order of numerator
%
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%

%
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
% Calculate poles, zeros and maximum radius.
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%
poles=roots([1 , thetaoe(n-1,1:na)]);
po_rad=abs(poles);
po_ang=angle(poles);
zeros_=roots([0 , thetaoe(n-1,na+1:na+nb)]);
ze_rad=abs(zeros_);
ze_ang=angle(zeros_);
max_radius=max([1 ; abs(poles) ; abs(zeros_)]);
%
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
% Calculate vector containing unit circle samples.
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%
w=(-pi:2*pi/512:pi);
unit_circle=1+0*w;
%
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
% Calculate vector containing axis.
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%
ang=[0 0 0 pi/2 pi/2 pi/2 pi pi pi -pi/2 -pi/2 -pi/2];
rad=[0 1 0 0 1 0 0 1 0 0 1 0];
rad_max=10*max_radius*rad;
%
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
% Plot overview out to max_radius*1.1.
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%
figure
hold on;
axis(max_radius*[-1.1 1.1 -1.1 1.1]);
polar(w,unit_circle,'-');
polar(po_ang,po_rad,'x');
polar(ze_ang,ze_rad,'o');
polar(ang,rad_max,'-');
axis equal;
hold off;
%
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
% Plot detailed picture of the interior of the unit circle.
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%
figure
axis([-1 1 -1 1]);
hold on;
polar(w,unit_circle,'-');
polar(po_ang,po_rad,'x');
polar(ze_ang,ze_rad,'o');
polar(ang,rad,'-');
axis equal;
hold off;
%
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
% Clear variables that are not used by subsequent programs.
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%
clear po_rad;
clear po_ang;
clear ze_rad;
clear ze_ang;
clear max_radius;
clear w;
clear unit_circle;
clear ang;
clear rad;
clear rad_max;